@@ -1,23 +1,20 @@
 /*
- * Copyright (C) 2014 Glyptodon LLC
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
  *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
+ *   http://www.apache.org/licenses/LICENSE-2.0
  *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
- * THE SOFTWARE.
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 
 /**
@@ -211,6 +208,53 @@ angular.module('client').directive('guacClient', [function guacClient() {
 
             };
 
+            /**
+             * Handles a mouse event originating from the user's actual mouse.
+             * This differs from handleEmulatedMouseState() in that the
+             * software mouse cursor must be shown only if the user's browser
+             * does not support explicitly setting the hardware mouse cursor.
+             *
+             * @param {Guacamole.Mouse.State} mouseState
+             *     The current state of the user's hardware mouse.
+             */
+            var handleMouseState = function handleMouseState(mouseState) {
+
+                // Do not attempt to handle mouse state changes if the client
+                // or display are not yet available
+                if (!client || !display)
+                    return;
+
+                // Send mouse state, show cursor if necessary
+                display.showCursor(!localCursor);
+                sendScaledMouseState(mouseState);
+
+            };
+
+            /**
+             * Handles a mouse event originating from one of Guacamole's mouse
+             * emulation objects. This differs from handleMouseState() in that
+             * the software mouse cursor must always be shown (as the emulated
+             * mouse device will not have its own cursor).
+             *
+             * @param {Guacamole.Mouse.State} mouseState
+             *     The current state of the user's emulated (touch) mouse.
+             */
+            var handleEmulatedMouseState = function handleEmulatedMouseState(mouseState) {
+
+                // Do not attempt to handle mouse state changes if the client
+                // or display are not yet available
+                if (!client || !display)
+                    return;
+
+                // Ensure software cursor is shown
+                display.showCursor(true);
+
+                // Send mouse state, ensure cursor is visible
+                scrollToMouse(mouseState);
+                sendScaledMouseState(mouseState);
+
+            };
+
             // Attach any given managed client
             $scope.$watch('client', function attachManagedClient(managedClient) {
 
@@ -238,6 +282,9 @@ angular.module('client').directive('guacClient', [function guacClient() {
                     return false;
                 };
 
+                // Size of newly-attached client may be different
+                $scope.mainElementResized();
+
             });
 
             // Update actual view scrollLeft when scroll properties change
@@ -264,20 +311,8 @@ angular.module('client').directive('guacClient', [function guacClient() {
             });
 
             // Swap mouse emulation modes depending on absolute mode flag
-            $scope.$watch('client.clientProperties.emulateAbsoluteMouse', function(emulateAbsoluteMouse) {
-
-                if (!client || !display) return;
-
-                var handleMouseState = function handleMouseState(mouseState) {
-
-                    // Ensure software cursor is shown
-                    display.showCursor(true);
-
-                    // Send mouse state, ensure cursor is visible
-                    scrollToMouse(mouseState);
-                    sendScaledMouseState(mouseState);
-
-                };
+            $scope.$watch('client.clientProperties.emulateAbsoluteMouse',
+                function mouseEmulationModeChanged(emulateAbsoluteMouse) {
 
                 var newMode, oldMode;
 
@@ -296,13 +331,17 @@ angular.module('client').directive('guacClient', [function guacClient() {
                 // Set applicable mouse emulation object, unset the old one
                 if (newMode) {
 
+                    // Clear old handlers and copy state to new emulation mode
                     if (oldMode) {
                         oldMode.onmousedown = oldMode.onmouseup = oldMode.onmousemove = null;
                         newMode.currentState.x = oldMode.currentState.x;
                         newMode.currentState.y = oldMode.currentState.y;
                     }
 
-                    newMode.onmousedown = newMode.onmouseup = newMode.onmousemove = handleMouseState;
+                    // Handle emulated events only from the new emulation mode
+                    newMode.onmousedown =
+                    newMode.onmouseup   =
+                    newMode.onmousemove = handleEmulatedMouseState;
 
                 }
 
@@ -357,21 +396,16 @@ angular.module('client').directive('guacClient', [function guacClient() {
 
             };
 
-            // Watch for changes to mouse emulation mode
-            // Send all received mouse events to the client
-            mouse.onmousedown =
-            mouse.onmouseup   =
-            mouse.onmousemove = function(mouseState) {
-
-                if (!client || !display)
-                    return;
-
-                // Send mouse state, show cursor if necessary
-                display.showCursor(!localCursor);
-                sendScaledMouseState(mouseState);
-
+            // Ensure focus is regained via mousedown before forwarding event
+            mouse.onmousedown = function(mouseState) {
+                document.body.focus();
+                handleMouseState(mouseState);
             };
 
+            // Forward mouseup / mousemove events untouched
+            mouse.onmouseup   =
+            mouse.onmousemove = handleMouseState;
+
             // Hide software cursor when mouse leaves display
             mouse.onmouseout = function() {
                 if (!display) return;
@@ -379,9 +413,11 @@ angular.module('client').directive('guacClient', [function guacClient() {
             };
 
             // Update remote clipboard if local clipboard changes
-            $scope.$on('guacClipboard', function onClipboard(event, mimetype, data) {
-                if (client)
-                    client.setClipboard(data);
+            $scope.$on('guacClipboard', function onClipboard(event, data) {
+                if (client) {
+                    ManagedClient.setClipboard($scope.client, data);
+                    $scope.client.clipboardData = data;
+                }
             });
 
             // Translate local keydown events to remote keydown events if keyboard is enabled
